@@ -21,6 +21,7 @@ __all__ = [
 ]
 
 import contextlib
+import warnings
 from typing import (
     Iterable,
     List,
@@ -31,21 +32,26 @@ from typing import (
 import optuna
 
 try:
-    # neptune-client=0.9.0+ package structure
-    import neptune.new as neptune
-    from neptune.new.integrations.utils import (
-        expect_not_an_experiment,
-        verify_type,
-    )
-    from neptune.new.types import File
+    # neptune-client<1.0.0 package structure
+    with warnings.catch_warnings():
+        # ignore the deprecation warnings
+        warnings.simplefilter("ignore")
+        import neptune.new as neptune
+        from neptune.new.integrations.utils import (
+            expect_not_an_experiment,
+            verify_type,
+        )
+        from neptune.new.types import File
+        from neptune.new.utils import stringify_unsupported
 except ImportError:
-    # neptune-client>=1.0.0 package structure
+    # neptune>=1.0.0 package structure
     import neptune
     from neptune.integrations.utils import (
         expect_not_an_experiment,
         verify_type,
     )
     from neptune.types import File
+    from neptune.utils import stringify_unsupported
 
 from neptune_optuna.impl.version import __version__
 
@@ -127,7 +133,7 @@ class NeptuneCallback:
 
     def __init__(
         self,
-        run: neptune.Run,
+        run: Union[neptune.Run, neptune.handler.Handler],
         base_namespace: str = "",
         plots_update_freq: Union[int, str] = 1,
         study_update_freq: Union[int, str] = 1,
@@ -144,7 +150,7 @@ class NeptuneCallback:
     ):
 
         expect_not_an_experiment(run)
-        verify_type("run", run, neptune.Run)
+        verify_type("run", run, (neptune.Run, neptune.handler.Handler))
         verify_type("base_namespace", base_namespace, str)
         verify_type("log_plots_freq", plots_update_freq, (int, str, type(None)))
         verify_type("log_study_freq", study_update_freq, (int, str, type(None)))
@@ -171,7 +177,11 @@ class NeptuneCallback:
         )
         verify_type("target_names", target_names, (list, type(None)))
 
-        self.run = run[base_namespace]
+        if base_namespace != "":
+            self.run = run[base_namespace]
+        else:
+            self.run = run
+
         self._visualization_backend = visualization_backend
         self._plots_update_freq = plots_update_freq
         self._study_update_freq = study_update_freq
@@ -186,7 +196,11 @@ class NeptuneCallback:
         self._target_names = target_names
         self._namespaces = None
 
-        run[INTEGRATION_VERSION_KEY] = __version__
+        root_obj = self.run
+        if isinstance(self.run, neptune.handler.Handler):
+            root_obj = self.run.get_root_object()
+
+        root_obj[INTEGRATION_VERSION_KEY] = __version__
 
     def __call__(self, study: optuna.Study, trial: optuna.trial.FrozenTrial):
         if self._namespaces is None:
@@ -202,7 +216,7 @@ class NeptuneCallback:
         _log_single_trial(self.run, study, trial=trial, namespaces=self._namespaces)
 
     def _log_trial_distributions(self, trial):
-        self.run["study/distributions"].log(trial.distributions)
+        self.run["study/distributions"] = stringify_unsupported(trial.distributions)
 
     def _log_best_trials(self, study):
         _log_best_trials(self.run, study, namespaces=self._namespaces)
@@ -405,7 +419,8 @@ def log_study_metadata(
         _log_trials(run, study, study.trials, namespaces=namespaces)
 
     if log_distributions:
-        run["study/distributions"].log([trial.distributions for trial in study.trials])
+        for i, trial in enumerate(study.trials):
+            run[f"study/distributions/trial/{i}"] = stringify_unsupported(trial.distributions)
 
     if log_plots:
         _log_plots(
@@ -445,7 +460,7 @@ def load_study_from_run(run: neptune.Run):
     Examples:
         Initialize an existing run by passing the run ID:
         >>> import neptune.new as neptune
-        ... run = neptune.init_run(run="PRO-123")
+        ... run = neptune.init_run(with_id="PRO-123")
 
         Load study from the run and continue optimization:
         >>> import neptune_optuna.impl as optuna_utils
@@ -472,11 +487,10 @@ def _log_study_details(run, study: optuna.Study):
     else:
         run["study/direction"] = study.direction
 
-    run["study/system_attrs"] = study.system_attrs
     run["study/user_attrs"] = study.user_attrs
     with contextlib.suppress(AttributeError):
         run["study/_study_id"] = study._study_id
-        run["study/_storage"] = study._storage
+        run["study/_storage"] = stringify_unsupported(study._storage)
 
 
 def _log_study(run, study: optuna.Study):
@@ -603,32 +617,33 @@ def _log_single_trial(run, study: optuna.Study, trial: optuna.trial.FrozenTrial,
 
     handle[f"trials/{trial._trial_id}/datetime_start"] = trial.datetime_start
     handle[f"trials/{trial._trial_id}/datetime_complete"] = trial.datetime_complete
-    handle[f"trials/{trial._trial_id}/duration"] = trial.duration
-    handle[f"trials/{trial._trial_id}/distributions"] = trial.distributions
-    handle[f"trials/{trial._trial_id}/intermediate_values"] = trial.intermediate_values
-    handle[f"trials/{trial._trial_id}/params"] = trial.params
+    handle[f"trials/{trial._trial_id}/duration"] = stringify_unsupported(trial.duration)
+    handle[f"trials/{trial._trial_id}/distributions"] = stringify_unsupported(trial.distributions)
+    handle[f"trials/{trial._trial_id}/intermediate_values"] = stringify_unsupported(trial.intermediate_values)
+    handle[f"trials/{trial._trial_id}/params"] = stringify_unsupported(trial.params)
 
     if _is_multi_objective(study=study):
-        handle[f"trials/{trial._trial_id}/values"] = {f"{namespaces[k]}": v for k, v in enumerate(trial.values)}
+        for k, v in enumerate(trial.values):
+            handle[f"trials/{trial._trial_id}/values/{namespaces[k]}"] = stringify_unsupported(v)
         if best:
-            handle["params"] = trial.params
+            handle["params"] = stringify_unsupported(trial.params)
             for k, v in enumerate(trial.values):
-                handle[f"values/{namespaces[k]}"] = v
+                handle[f"values/{namespaces[k]}"] = stringify_unsupported(v)
         else:
-            handle["params"].log(trial.params)
+            handle["params"].append(stringify_unsupported(trial.params))
             for k, v in enumerate(trial.values):
-                handle[f"values/{namespaces[k]}"].log(v, step=trial._trial_id)
+                handle[f"values/{namespaces[k]}"].append(stringify_unsupported(v), step=trial._trial_id)
 
     else:
-        handle[f"trials/{trial._trial_id}/value"] = trial.value
+        handle[f"trials/{trial._trial_id}/value"] = stringify_unsupported(trial.value)
         if best:
-            handle["value"] = trial.value
-            handle["params"] = trial.params
+            handle["value"] = stringify_unsupported(trial.value)
+            handle["params"] = stringify_unsupported(trial.params)
             handle["value|params"] = f"value: {trial.value}| params: {trial.params}"
         else:
-            handle["values"].log(trial.value, step=trial._trial_id)
-            handle["params"].log(trial.params)
-            handle["values|params"].log(f"value: {trial.value}| params: {trial.params}")
+            handle["values"].append(stringify_unsupported(trial.value), step=trial._trial_id)
+            handle["params"].append(stringify_unsupported(trial.params))
+            handle["values|params"].append(f"value: {trial.value}| params: {trial.params}")
 
     if trial.state.is_finished() and trial.state != optuna.trial.TrialState.COMPLETE:
         handle[f"trials/{trial._trial_id}/state"] = repr(trial.state)
